<?php

namespace SGalinski\SgMail\Service;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use TYPO3\CMS\Core\Cache\CacheManager;
use TYPO3\CMS\Core\Cache\Frontend\FrontendInterface;
use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Object\ObjectManager;

/**
 * Provides an api for registering your mail templates
 */
class RegisterService implements \TYPO3\CMS\Core\SingletonInterface {
	const CACHE_NAME = 'sg_mail_registerArrayCache';
	const CACHE_LIFETIME_IN_SECONDS = 86400;
	const DEFAULT_TEMPLATE_PATH = 'Resources/Private/Templates/SgMail/';
	const CONFIG_PATH = 'Configuration/MailTemplates/Forms';

	/**
	 * contains the actual registration data
	 *
	 * @var array
	 */
	private $registerArray = [];

	/**
	 * Get all registered templates from the cache. if the cache expired, the entries are generated
	 *
	 * @return array
	 * @throws \TYPO3\CMS\Core\Cache\Exception\NoSuchCacheException
	 * @throws \BadFunctionCallException
	 * @throws \InvalidArgumentException
	 */
	public function getRegisterArray(): array {
		$cacheManager = GeneralUtility::makeInstance(CacheManager::class);
		$cache = $cacheManager->getCache(self::CACHE_NAME);
		$cacheId = 'sg_mail';

		$entry = $cache->get($cacheId);
		if (!$entry) {
			$this->clearCaches();
			$entry = $this->registerExtensions();

			if ($entry === NULL) {
				$entry = [];
			}

			$cache->set($cacheId, $entry, [], self::CACHE_LIFETIME_IN_SECONDS);
		}

		return $entry;
	}

	/**
	 * Read every registered file and create a registration entry in the registerArray if possible
	 *
	 * @return array
	 */
	private function registerExtensions(): array {
		$this->registerArray = [];

		$extensionConfiguration = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['sg_mail'], []);

		// @TODO remove in version 5.0.0
		if ($extensionConfiguration['enableSecurity'] === 0) {
			MailTemplateService::registerExtensions();
		}

		foreach ($GLOBALS['sgmail'] as $extension) {
			foreach ($extension as $key => $registerFile) {
				if (\is_file($registerFile)) {
					$configArray = (include $registerFile);

					$extensionKey = $configArray['extension_key'];
					$templateKey = $configArray['template_key'];

					if ($extensionKey === NULL || $templateKey === NULL) {
						continue;
					}

					$this->writeRegisterArrayEntry(
						$extensionKey, $templateKey, $configArray
					);
				}
			}
		}

		// read automatically created extension registrations and write the entries
		$configurationLocation = $this->getRegistrationPath();
		$registerFolder = GeneralUtility::getFileAbsFileName(
			$configurationLocation
		);
		if (\is_dir($registerFolder)) {
			$configFiles = GeneralUtility::getFilesInDir($registerFolder);
			foreach ($configFiles as $configFile) {
				$pathToRegistrationFile = $registerFolder . '/' . $configFile;

				if (!\file_exists($pathToRegistrationFile)) {
					continue;
				}

				// get file name without folders
				$pathAsArray = GeneralUtility::trimExplode('/', $pathToRegistrationFile);
				$filename = $pathAsArray[\count($pathAsArray) - 1];
				$filenameWithoutHash = GeneralUtility::trimExplode('_', $filename)[1];
				$hash = md5($GLOBALS['TYPO3_CONF_VARS']['SYS']['encryptionKey'] . '|' . $filenameWithoutHash);
				// if the filename doesnt start with the hash value, ignore it
				if (strpos($filename, $hash) !== 0) {
					continue;
				}

				$configArray = (include $pathToRegistrationFile);

				$extensionKey = $configArray['extension_key'];
				$templateKey = $configArray['template_key'];

				if ($extensionKey === NULL || $templateKey === NULL) {
					continue;
				}

				$this->writeRegisterArrayEntry(
					$extensionKey, $templateKey, $configArray
				);
			}
		}

		return $this->registerArray;
	}

	/**
	 * write a single entry into the register array
	 *
	 * @param string $extensionKey
	 * @param string $templateKey
	 * @param array $configArray
	 * @param bool $transformTemplateFolder
	 * @param string $storeTemplateExtension
	 */
	private function writeRegisterArrayEntry(
		$extensionKey, $templateKey, array $configArray,
		$transformTemplateFolder = TRUE, $storeTemplateExtension = ''
	) {
		// If it is not explicitly set in which extension the html should be located, use the extension set in the template settings
		if ($storeTemplateExtension === '') {
			$storeTemplateExtension = $extensionKey;
		}

		// give the option to use the template key as folder name. this is used mainly with auto registering
		$templateDirectory = $templateKey;

		// by default folders with underscore will be transformed to upper camelcase
		if ($transformTemplateFolder) {
			// transform template directory name: your_templates => YourTemplates/
			$templateDirectoryParts = GeneralUtility::trimExplode('_', $templateKey);
			$templateDirectory = '';
			foreach ($templateDirectoryParts as $part) {
				$templateDirectory .= \ucfirst($part);
			}
		}

		$templateDirectory .= '/';
		$templatePath = ExtensionManagementUtility::extPath($storeTemplateExtension) . self::DEFAULT_TEMPLATE_PATH
			. $templateDirectory;

		if ($configArray['template_path']) {
			$templatePath = $configArray[$templateKey];
		}

		$description = $configArray['description'];
		$subject = $configArray['subject'];
		$marker = $configArray['markers'];
		$templateContent = $configArray['templateContent'];

		$this->registerArray[$extensionKey][$templateKey] = [
			'templatePath' => $templatePath,
			'description' => $description,
			'marker' => $marker,
			'extension' => $extensionKey,
			'templateName' => $templateKey,
			'subject' => $subject,
			'templateContent' => $templateContent
		];
	}

	/**
	 * Returns the path to the configured location where automatic mail template registrations should be
	 *
	 * @return string
	 */
	private function getRegistrationPath(): string {
		// get typoscript settings from sg mail
		/** @var TypoScriptSettingsService $typoScriptSettingsService */
		$typoScriptSettingsService = GeneralUtility::makeInstance(TypoScriptSettingsService::class);
		$tsSettings = $typoScriptSettingsService->getSettings(0, 'tx_sgmail');

		// get the location where automatic registrations should be stored
		return 'EXT:' . $tsSettings['mail']['configurationLocation'] . '/' . self::CONFIG_PATH;
	}

	/**
	 * Clear the sgmail register cache
	 *
	 * @throws \TYPO3\CMS\Core\Cache\Exception\NoSuchCacheException
	 */
	private function clearCaches() {
		$objectManager = GeneralUtility::makeInstance(ObjectManager::class);
		$cacheManager = $objectManager->get(CacheManager::class);

		/** @var FrontendInterface $cache */
		$cache = $cacheManager->getCache(self::CACHE_NAME);
		/** @var FrontendInterface $cache */
		$cache->flush();
	}
}

